/*
Flux2D - algorithme de simulation pour SimuApplet

Copyright (C) 2002 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.awt.image.PixelGrabber;
import java.net.URL;
import java.net.MalformedURLException;
import simu.*;

/**
 * Algorithme de simulation pour SimuApplet.
 * Flux2D.
 */
//	compilation: javac -target 1.1 -bootclasspath ~/Java/jdk1.1/JDKClasses.zip -extdirs "" -encoding ISO-8859-1 -classpath .:SimuLab.jar Flux2D.java
public class Flux2D extends SimuApplet {
	
	final static int npts = 128;
	final static double lambda = 0.001;
	final static int tmax = 100000;
	final static String nomImage = "Kolmogorov.png";
	static int[][] lambdi = null; // valeurs de l'image initiale
	static int w,h;
    Affichage aff;
	double vfact;
    double ta[][];
    double ts[][];
	double moyenne;
	long ct;
	int nt;

    public Flux2D(){
        
        // paramtres d'entre
        ParamIn vitParam = new ParamIn();
        vitParam.type = "nombre";
        vitParam.label = "vitesse";
        vitParam.titre = "vitesse";
        vitParam.acquisition = "champ";
        vitParam.defaut = "1000";
        vitParam.unit = "photons/itration";
        addParamIn(vitParam);
		
        // paramtres de sortie
        ParamOut tsp = new ParamOut();
        tsp.type = "tableau";
        tsp.label = "ts";
        addParamOut(tsp);

		// infos (n itrations)
        ParamOut info = new ParamOut();
        info.type = "string";
        info.label = "info";
        addParamOut(info);
		
        // options d'affichage
		appletDoc("Flux2D_doc.xml");
		
        aff = new Affichage();
        aff.type = "image";
        aff.titre = "Flux 2D";
        String[][] pparams = {{"ts"}};
        aff.params = pparams;
        addAffichage(aff);

}

	private void lectureImage() {
		Image img;
		URL hurle;
		URL base = null;
		try {
			base = getCodeBase();
		} catch (Exception ex) {
			System.err.println(ex.getClass().getName() + ": " + ex.getMessage());
		}
		if (base == null) {
			img = Toolkit.getDefaultToolkit().getImage(nomImage);
		} else {
			try {
				hurle = new URL(base, nomImage);
			} catch (MalformedURLException ex) {
				System.err.println(ex.getClass().getName() + ": " + ex.getMessage());
				return;
			}
			img = getImage(hurle);
		}
		MediaTracker tracker = new MediaTracker(this);
		tracker.addImage(img, 0);
		try {
			tracker.waitForID(0);
		} catch (Exception ex) { }
		
		w = img.getWidth(null);
		h = img.getHeight(null);
		//System.out.println("h="+h+" w="+w);
		if (w == -1 || h == -1)
			System.err.println("taille>0: Erreur au chargement de l'image");
		
		int[] pixels = new int[w * h];
		PixelGrabber pg = new PixelGrabber(img, 0, 0, w, h, pixels, 0, w); 
		try {
			pg.grabPixels();
		} catch (InterruptedException e) {
			System.err.println("interrupted waiting for pixels!");
			return;
		}
		lambdi = new int[w][h];
		int pix,red;
		for (int iy=0; iy<h; iy++)
			for (int ix=0; ix<w; ix++) {
				pix = pixels[iy*w + ix];
				red = (pix >> 16) & 0xFF;
				lambdi[ix][iy] = red;
			}
	}
	
// calcul en image (utilis avec affichage image)
    public void initCalculLive(ListeValeurs in) throws SimuException {
		
		if (lambdi == null)
			lectureImage();
		
        aff.imgdimx = w;
        aff.imgdimy = h;
		
		double vitesse = in.lireDouble("vitesse");
		double somme = 0;
        for(int i=0; i<w; i++)
            for(int j=0; j<h; j++)
				somme += lambdi[i][j];
		vfact = vitesse/somme; // le dlai par dfaut entre 2 itrations est 100ms (quand le calcul est rapide)
		
        ta = new double[w][h];
        ts = new double[w][h];
		
        for(int i=0; i<w; i++)
            for(int j=0; j<h; j++)
				ta[i][j] = 0;
		
		moyenne = 0;
		
		nt = 0;
		ct = System.currentTimeMillis();
    }
    
    public ListeValeurs calculLive() throws SimuException {
        if (nt >= tmax)
            return null; // fin du calcul
		
			nt++;
			
		double mu;
		int num;
        for(int i=0; i<w; i++)
            for(int j=0; j<h; j++) {
				mu = ((double)lambdi[i][j])*vfact;
				num = Proba.sPoisson1(mu);
				ta[i][j] += num;
				moyenne += ((double)num)/(h*w);
			}
		
		double v;
		double moy2 = 128/moyenne;
        for (int i=0; i<w; i++)
            for (int j=0; j<h; j++) {
				v = ta[i][j] * moy2;
				if (v > 255)
					v = 255;
				ts[i][j] = v;
			}
		
        ListeValeurs out = new ListeValeurs();
		
		out.ajouterString("info", "nb d'itrations: " + nt + "  moyenne=" + moyenne);
		
        out.ajouter("ts", ts);
		
        return out;
    }
    
    public ListeValeurs calcul(ListeValeurs in) throws SimuException {
        throw new SimuException("calcul n'est pas implment dans Flux2D: " +
            "utiliser calculLive  la place");
    }

}
