// utilise Proba

/**
  * Utilitaires pour les dmos
  */
public class MoreProba {

  /**
    * best box for the repartition function
    * [0] : xmin
    * [1] : xmax
    * [2] : ymin
    * [3] : ymax
	*/
  public static double[] bestBoxF(double mu) {
    final double fact = 3.5;
	double[] box = new double[4];

    box[0] = Math.max(-0.5, Math.floor(mu - fact*Math.sqrt(mu)));
    box[1] = Math.ceil(mu + fact*Math.sqrt(mu)) + 0.5;
    box[2] = 0.0;
    box[3] = 1.01;
	
	return(box);
  }

  /**
    * best box for the Sample
    * [0] : xmin
    * [1] : xmax
    * [2] : ymin
    * [3] : ymax
	*/
  public static double[] bestBoxS(double mu, int npts) {
	double[] box = new double[4];
	
    box[0] = 0.5;
    box[1] = npts + 0.5;
    box[2] = Math.max(0.0, mu - 5.*Math.sqrt(mu));
    box[3] = Math.ceil(mu + 5.*Math.sqrt(mu)) + 0.1;
	
	return(box);
  }

  /**
    * best box for the Histogram
    * [0] : xmin
    * [1] : xmax
    * [2] : ymin
    * [3] : ymax
	*/
  public static double[] bestBoxH(double mu, int npts) {
    final double fact = 3.5;
	double proba;
	double[] box = new double[4];

    box[0] = Math.max(-0.5, Math.floor(mu - 3.5*Math.sqrt(mu)));
    box[1] = Math.max(10., Math.ceil(mu + 3.5*Math.sqrt(mu))) + 0.5;
    box[2] = 0.0;
	try {
    	proba = Proba.pPoisson(mu, (int)Math.floor(mu));
	} catch (Exception ex) {
		// on ignore l'erreur ?
		proba = 0;
	}
    box[3] = npts*proba + fact*Math.sqrt(npts*proba*(1.0 - proba));
	
	return(box);
  }

  /**
    * best box for PFEdemo2 (Erlang...)
    * [0] : xmin
    * [1] : xmax
    * [2] : ymin
    * [3] : ymax
	*/
  public static double[] bestBox2(double lambda, int k) {
    final double fact = 3.5;
	double[] box = new double[4];
	
	double nnu = k;
	double sigma = Math.sqrt(nnu/(lambda*lambda));
	
    box[0] = Math.max(0.0, nnu/lambda - fact*sigma);
    box[1] = nnu/lambda + fact*sigma;
    box[2] = 0.0;
    box[3] = 1.001;
	
	return(box);
  }

  /**
    * valeur maximale d'un tableau d'entiers
	*/
  public static int maxval(int[] t) {
    int max = t[0];
  	for (int i=0; i<t.length; i++)
		if (t[i] > max)
			max = t[i];
	return(max);
  }

  /**
    * valeur minimale d'un tableau d'entiers
	*/
  public static int minval(int[] t) {
    int min = t[0];
  	for (int i=0; i<t.length; i++)
		if (t[i] < min)
			min = t[i];
	return(min);
  }
  
  public static double[] tableauDouble(int[] t) {
    double[] dt = new double[t.length];
	for (int i=0; i<t.length; i++)
	    dt[i] = t[i];
	return(dt);
  }
  
  /* on peut utiliser clone()  la place
  public static int[] copieTableau(int[] t) {
    int[] t2 = new int[t.length];
	for (int i=0; i<t.length; i++)
	    t2[i] = t[i];
	return(t2);
  }
  */
  
  public static double[] doHistoSample(double[] y, double ymin, double ymax, int nh) {
    double Dy, limit;
    double histup, histdown;

	double[] histo = new double[nh];
    Dy = (ymax - ymin)/(nh - 1);
    histdown = 0;
    for (int i=0; i<nh; i++) {
       limit = ymin + Dy*i;
	   histup = 0;
	   for (int j=0; j<y.length; j++)
	       if (y[j] <= limit)
		       histup++;
       histo[i] = histup - histdown;
       histdown = histup;
    }
	return(histo);
  }

}
